@@ -1,11 +1,16 @@
 package SReview::Talk;
 
 use Moose;
+
+use Media::Convert::Asset;
+use Media::Convert::Asset::ProfileFactory;
 use Mojo::Pg;
 use Mojo::Template;
 use Mojo::JSON qw/encode_json decode_json/;
 use SReview::Config::Common;
 use SReview::Talk::State;
+use SReview::Talk::Progress;
+use DateTime::Format::Pg;
 
 my $config = SReview::Config::Common::setup;
 my $pg = Mojo::Pg->new->dsn($config->get('dbistring')) or die "Cannot connect to database!";
@@ -56,6 +61,17 @@ has 'talkid' => (
         },
 );
 
+has 'upstreamid' => (
+	is => 'ro',
+	isa => 'Str',
+	builder => '_probe_upstreamid',
+	lazy => 1,
+);
+
+sub _probe_upstreamid {
+	return shift->_get_pathinfo->{raw}{upstreamid}
+}
+
 =head2 pathinfo
 
 Helper property to look up information from the database. Should not be
@@ -75,12 +91,10 @@ sub _load_pathinfo {
 
 	my $pathinfo = {};
 
-	my $eventdata = $pg->db->dbh->prepare("SELECT events.id AS eventid, events.name AS event, events.outputdir AS event_output, rooms.name AS room, rooms.outputname AS room_output, rooms.id AS room_id, talks.starttime::date AS date, to_char(starttime, 'DD Month yyyy at HH:MI') AS readable_date, to_char(starttime, 'yyyy') AS year, talks.slug, talks.title, talks.subtitle, talks.state, talks.nonce, talks.apologynote FROM talks JOIN events ON talks.event = events.id JOIN rooms ON rooms.id = talks.room WHERE talks.id = ?");
+	my $eventdata = $pg->db->dbh->prepare("SELECT events.id AS eventid, events.name AS event, events.outputdir AS event_output, rooms.name AS room, rooms.outputname AS room_output, rooms.id AS room_id, talks.starttime, talks.starttime::date AS date, to_char(starttime, 'DD Month yyyy at HH:MI') AS readable_date, to_char(talks.starttime, 'yyyy') AS year, talks.endtime, talks.slug, talks.title, talks.subtitle, talks.state, talks.progress, talks.nonce, talks.apologynote, talks.upstreamid FROM talks JOIN events ON talks.event = events.id JOIN rooms ON rooms.id = talks.room WHERE talks.id = ?");
 	$eventdata->execute($self->talkid);
 	my $row = $eventdata->fetchrow_hashref();
 
-	$pathinfo->{"workdir"} = join('/', $row->{eventid}, $row->{date}, substr($row->{room}, 0, 1));
-
 	my @elements = ($config->get('outputdir'));
 	foreach my $element(@{$config->get('output_subdirs')}) {
 		push @elements, $row->{$element};
@@ -175,7 +189,7 @@ has 'comment' => (
 
 sub _load_comment {
         my $self = shift;
-        my $st = $pg->db->dbh->prepare("SELECT comments FROM talks WHERE id = ?");
+        my $st = $pg->db->dbh->prepare("WITH orderedlog(talk, comment, logdate) AS (SELECT talk, comment, logdate FROM commentlog ORDER BY logdate DESC) SELECT talk, string_agg(logdate || E'\n' || comment, E'\n\n') AS comments FROM orderedlog WHERE talk = ? GROUP BY talk");
         $st->execute($self->talkid);
         my $row = $st->fetchrow_hashref;
         return $row->{comments};
@@ -309,6 +323,24 @@ sub _load_event_output {
 	return shift->_get_pathinfo->{raw}{event_output};
 }
 
+=head2 progress
+
+The current progress value of the talk, as an L<SReview::Talk::Progress>
+
+=cut
+
+has 'progress' => (
+	lazy => 1,
+	is => 'rw',
+	isa => 'SReview::Talk::Progress',
+	builder => '_load_progress',
+);
+
+sub _load_progress {
+	my $self = shift;
+	return SReview::Talk::Progress->new($self->_get_pathinfo->{raw}{progress});
+}
+
 =head2 state
 
 The current state of the talk, as an L<SReview::Talk::State>
@@ -375,7 +407,7 @@ has 'workdir' => (
 
 sub _load_workdir {
 	my $self = shift;
-	return join('/', $config->get("pubdir"), $self->_get_pathinfo->{"workdir"});
+	return join('/', $config->get("pubdir"), $self->relative_name);
 }
 
 =head2 relative_name
@@ -393,7 +425,12 @@ has 'relative_name' => (
 
 sub _load_relative_name {
 	my $self = shift;
-	return join('/', $self->_get_pathinfo->{"workdir"}, $self->_get_pathinfo->{'slug'});
+	my $n = $self->nonce;
+	my $serial = $self->has_correction("serial") ? ${$self->corrections}{serial} : 0;
+	if($self->state eq "broken" && $serial > 0) {
+		$serial -= 1;
+	}
+	return join('/', substr($n, 0, 1), substr($n, 1, 2), substr($n, 3), $serial);
 }
 
 =head2 outname
@@ -504,16 +541,12 @@ sub _load_corrections {
 		$corrections{$name} = $val;
 	}
 
-	foreach my $prop ("offset_start", "length_adj", "offset_audio") {
+	foreach my $prop ("offset_start", "length_adj", "offset_audio", "audio_channel") {
 		if(!exists($corrections{$prop})) {
 			$corrections{$prop} = 0;
 		}
 	}
 
-	if(!exists($corrections{audio_channel})) {
-		$corrections{audio_channel} = 0;
-	}
-
 	return \%corrections;
 }
 
@@ -578,6 +611,37 @@ sub _load_video_fragments {
 	return $rows;
 }
 
+=head2 avs_video_fragments
+
+The same values as the video_fragments attribute, but with every length
+extended as needed for A/V sync operations.
+
+=cut
+
+has 'avs_video_fragments' => (
+	lazy => 1,
+	is => 'rw',
+	builder => '_load_avs_video_fragments',
+);
+
+sub _load_avs_video_fragments {
+	my $self = shift;
+	my $corrections = $self->corrections;
+
+	if($corrections->{offset_audio} == 0) {
+		return $self->video_fragments;
+	}
+	my $talk_data = $pg->db->dbh->prepare("SELECT talkid, rawid, raw_filename, extract(epoch from fragment_start) as fragment_start, extract(epoch from raw_length) as raw_length, extract(epoch from raw_length_corrected) as raw_length_corrected from adjusted_raw_talks(?, make_interval(secs :=?::numeric), make_interval(secs := ?::numeric), make_interval(secs :=abs(?::numeric))) order by talk_start, raw_start");
+	$talk_data->execute($self->talkid, $corrections->{"offset_start"}, $corrections->{"length_adj"}, $corrections->{"offset_audio"});
+
+	my $rows;
+	while(my $row = $talk_data->fetchrow_hashref()) {
+		push @$rows, $row;
+	}
+
+	return $rows;
+}
+
 =head2 speakers
 
 The names of the speakers as a single string, in the format 'Firstname
@@ -620,7 +684,7 @@ has 'speakerlist' => (
 sub _load_speakerlist {
 	my $self = shift;
 
-	my $query = $pg->db->dbh->prepare("SELECT speakers.name FROM speakers JOIN speakers_talks ON speakers.id = speakers_talks.speaker WHERE speakers_talks.talk = ?");
+	my $query = $pg->db->dbh->prepare("SELECT speakers.name FROM speakers JOIN speakers_talks ON speakers.id = speakers_talks.speaker WHERE speakers_talks.talk = ? ORDER BY speakers.name");
 
 	$query->execute($self->talkid);
 
@@ -683,7 +747,7 @@ sub _load_eventurl {
 	my $self = shift;
 	my $mt = Mojo::Template->new;
 	if(defined($config->get('eventurl_format'))) {
-		return $mt->vars(1)->render($config->get('eventurl_format'), {
+		my $rv = $mt->vars(1)->render($config->get('eventurl_format'), {
 			slug => $self->slug,
 			room => $self->room,
 			date => $self->date,
@@ -691,11 +755,52 @@ sub _load_eventurl {
 			event_output => $self->event_output,
 			talk => $self,
 			year => $self->_get_pathinfo->{raw}{year}});
+		chomp $rv;
+		return $rv;
 	}
 	return "";
 }
 
-=head2
+=head2 output_video_urls
+
+An array of URLs for the output videos, as they will be published. Used by final review.
+
+=cut
+
+has 'output_video_urls' => (
+	lazy => 1,
+	is => 'ro',
+	isa => 'ArrayRef[HashRef[Str]]',
+	builder => '_load_output_urls',
+);
+
+sub _load_output_urls {
+	my $self = shift;
+	my $mt = Mojo::Template->new;
+	my $form = $config->get("output_video_url_format");
+	my $rv = [];
+	if(defined($form)) {
+		my $vid = Media::Convert::Asset->new(url => "");
+		foreach my $prof(@{$config->get("output_profiles")}) {
+			my $item = {prof => $prof};
+			if($prof eq "copy") {
+				$prof = $config->get("input_profile");
+			}
+			my $exten = Media::Convert::Asset::ProfileFactory->create($prof, $vid, $config->get('extra_profiles'))->exten;
+			my $url = $mt->vars(1)->render($form, {
+				talk => $self,
+				year => $self->_get_pathinfo->{raw}{year},
+				exten => $exten
+			});
+			chomp $url;
+			$item->{url} = $url;
+			push @$rv, $item;
+		}
+	}
+	return $rv;
+}
+
+=head2 preview_exten
 
 The file extension of the preview file (.webm or .mp4)
 
@@ -712,6 +817,25 @@ sub _load_preview_exten {
 	return $config->get('preview_exten');
 }
 
+=head2 scheduled_length
+
+The length of the talk, as scheduled
+
+=cut
+
+has 'scheduled_length' => (
+	is => "ro",
+	lazy => 1,
+	builder => "_load_scheduled_length",
+);
+
+sub _load_scheduled_length {
+	my $self = shift;
+	my $start = DateTime::Format::Pg->parse_datetime($self->_get_pathinfo->{raw}{starttime});
+	my $end = DateTime::Format::Pg->parse_datetime($self->_get_pathinfo->{raw}{endtime});
+	return $end->epoch - $start->epoch;
+}
+
 =head1 METHODS
 
 =head2 by_nonce
@@ -741,9 +865,16 @@ Looks up (and returns) the talk by slug, rather than by talk ID
 sub by_slug {
 	my $klass = shift;
 	my $slug = shift;
+	my $event = shift;
 
-	my $st = $pg->db->dbh->prepare("SELECT * FROM talks WHERE slug = ?");
-	$st->execute($slug);
+	my $st;
+	if(defined($event)) {
+		$st = $pg->db->dbh->prepare("SELECT * FROM talks WHERE slug = ? AND event = ?");
+		$st->execute($slug, $event);
+	} else {
+		$st = $pg->db->dbh->prepare("SELECT * FROM talks WHERE slug = ?");
+		$st->execute($slug);
+	}
 	die "Talk does not exist (or the slug is not unique in the database).\n" unless $st->rows == 1;
 	my $row = $st->fetchrow_arrayref;
 	my $rv = SReview::Talk->new(talkid => $row->[0], slug => $slug);
@@ -793,9 +924,7 @@ sub done_correcting {
                 $st->execute($self->talkid, $pair->[0], $pair->[1]);
         }
         if($self->has_comment) {
-                $db->prepare("UPDATE talks SET comments=? WHERE id = ?")->execute($self->comment, $self->talkid);
-        } else {
-                $db->prepare("UPDATE talks SET comments = NULL WHERE id = ?")->execute($self->talkid);
+                $db->prepare("INSERT INTO commentlog(comment, talk, state) VALUES (?, ?, ?)")->execute($self->comment, $self->talkid, $self->state);
         }
 	if($self->has_apology) {
 		$db->prepare("UPDATE talks SET apologynote=? WHERE id = ?")->execute($self->apology, $self->talkid);
@@ -822,9 +951,10 @@ sub set_state {
 	my $progress = shift;
 
 	$progress = 'waiting' unless defined($progress);
+	my $dbh = $pg->db->dbh;
 
-        my $st = $pg->db->dbh->prepare("UPDATE talks SET state=?, progress=? WHERE id=?");
-        $st->execute($newstate, $progress, $self->talkid);
+        my $st = $dbh->prepare("UPDATE talks SET state=?, progress=? WHERE id=?") or die $dbh->errstr;
+        $st->execute($newstate, $progress, $self->talkid) or die $dbh->errstr;
 }
 
 =head2 state_done
