@@ -4,33 +4,42 @@ use Mojo::Base 'Mojolicious';
 use Mojo::Collection 'c';
 use Mojo::JSON qw(encode_json);
 use Mojo::Pg;
+use Mojo::URL;
+use Crypt::PRNG qw/random_string/;
 use SReview;
 use SReview::Config;
 use SReview::Config::Common;
 use SReview::Db;
-use SReview::Video;
-use SReview::Video::ProfileFactory;
+use Media::Convert::Asset;
+use Media::Convert::Asset::ProfileFactory;
+use SReview::API;
+use SReview::Files::Factory;
 
 sub startup {
 	my $self = shift;
 
 	my $dir = $ENV{SREVIEW_WDIR};
 
-	$self->config(hypnotoad => { pid_file => '/var/run/sreview/sreview-web.pid' });
-	$self->plugin('OAuth2::Server' => 
+	SReview::API::init($self);
 
 	my $config = SReview::Config::Common::setup;
+	$self->max_request_size(2*1024*1024*1024);
+
+	if(defined($config->get("web_pid_file"))) {
+		$self->config(hypnotoad => { pid_file => $config->get("web_pid_file") });
+	}
 
 	die "Need to configure secrets!" if $config->get("secret") eq "_INSECURE_DEFAULT_REPLACE_ME_";
 	$self->secrets([$config->get("secret")]);
 
 	SReview::Db::init($config);
 
-	if($self->mode eq "production") {
+	if(-d "/usr/share/sreview/templates") {
 		push @{$self->renderer->paths}, "/usr/share/sreview/templates";
 		push @{$self->static->paths}, "/usr/share/sreview/public";
 		push @{$self->static->paths}, "/usr/share/javascript";
-	} else {
+	}
+	if(-d "public") {
 		push @{$self->static->paths}, "./public";
 		push @{$self->renderer->paths}, "./templates";
 	}
@@ -41,18 +50,53 @@ sub startup {
 	$self->hook(before_dispatch => sub {
 		my $c = shift;
 		my $vpr = $config->get('vid_prefix');
-		my $media = "media-src 'self'";
-		if(defined($vpr) && length($vpr) > 0) {
-			$media = "media-src $vpr;";
+		state $media = undef;
+		if(!defined($media)) {
+			$media = "media-src 'self'";
+			my $url = Mojo::URL->new($vpr);
+			if(defined($url->host)) {
+				$vpr = $url->host;
+				$media = "media-src $vpr";
+			}
+			if(defined($config->get("finalhosts"))) {
+				$media .= " " . $config->get("finalhosts");
+			}
+			$media .= ";";
 		}
-		$c->res->headers->content_security_policy("default-src 'none'; connect-src 'self'; script-src 'self' 'unsafe-inline'; font-src 'self'; style-src 'self'; img-src 'self'; frame-ancestors 'none'; $media");
+		$c->res->headers->content_security_policy("default-src 'none'; connect-src 'self'; script-src 'self' 'unsafe-inline' 'unsafe-eval'; font-src 'self'; style-src 'self'; img-src 'self' data:; frame-ancestors 'none'; $media");
 	});
 
 	$self->helper(dbh => sub {
 		state $pg = Mojo::Pg->new->dsn($config->get('dbistring'));
 		return $pg->db->dbh;
 	});
-	
+	$self->helper(srconfig => sub {
+		return $config;
+	});
+
+	$self->helper(auth_scope => sub {
+		my $c = shift;
+		my $scope = shift;
+		$self->log->debug("checking if authorized for $scope");
+		if($c->session->{admin}) {
+			return 1;
+		} elsif($scope eq "api") {
+			return 1;
+		} elsif($scope eq "api/event") {
+			return 1;
+		} elsif($scope eq "api/talks") {
+			if(exists($c->session->{id})) {
+				return 1;
+			}
+		} elsif($scope eq "api/talks/detailed") {
+			if(exists($c->session->{id})) {
+				return 1;
+			}
+		}
+		$self->log->debug("not authorized for $scope");
+		return 0;
+	});
+
 	$self->helper(talk_update => sub {
 		my $c = shift;
 		my $talk = shift;
@@ -91,13 +135,22 @@ sub startup {
 	});
 
 	my $eventid = undef;
-	my $st = $self->dbh->prepare("SELECT id FROM events WHERE name = ?");
-	$st->execute($config->get("event")) or die "Could not find event!\n";
-	while(my $row = $st->fetchrow_hashref("NAME_lc")) {
-		die if defined($eventid);
-		$eventid = $row->{id};
-	}
 	$self->helper(eventid => sub {
+		if(!defined($eventid)) {
+			if(defined($config->get("event"))) {
+				my $st = $self->dbh->prepare("SELECT id FROM events WHERE name = ?");
+				$st->execute($config->get("event")) or die "Could not find event!\n";
+				while(my $row = $st->fetchrow_hashref("NAME_lc")) {
+					die if defined($eventid);
+					$eventid = $row->{id};
+				}
+			} else {
+				my $st = $self->dbh->prepare("SELECT max(id) FROM events");
+				$st->execute() or die "Could not query for events";
+				my $row = $st->fetchrow_hashref("NAME_lc");
+				$eventid = $row->{id};
+			}
+		}
 		return $eventid;
 	});
 
@@ -110,7 +163,15 @@ sub startup {
 		$rv = <GIT>;
 		close GIT;
 		if(!defined $rv) {
-			return $SReview::VERSION;
+			if(exists($ENV{GIT_DESCRIBE})) {
+				$rv = $ENV{GIT_DESCRIBE};
+			} else {
+				if(exists($ENV{OPENSHIFT_BUILD_COMMIT})) {
+					$rv = $SReview::VERSION . ", built from commit " . $ENV{OPENSHIFT_BUILD_COMMIT};
+				} else {
+					$rv = $SReview::VERSION;
+				}
+			}
 		}
 		chomp $rv;
 		return $rv;
@@ -151,66 +212,21 @@ sub startup {
 		if($c->session->{volunteer}) {
 			return $c->redirect_to('/volunteer/list');
 		} else {
-			return $c->redirect_to('/admin');
+			my $apikey = random_string();
+			$c->cookie(sreview_api_key => $apikey);
+			$c->session->{apikey} = $apikey;
+			return $c->redirect_to('/overview');
 		}
 	});
 
+	$r->get('/i/:nonce')->to(controller => 'inject', action => 'view', layout => 'default');
+	$r->post('/i/:nonce/update')->to(controller => 'inject', action => 'update', layout => 'default');
+
 	$r->get('/r/:nonce')->to(controller => 'review', action => 'view', layout => 'default');
 	$r->post('/r/:nonce/update')->to(controller => 'review', layout => 'default', action => 'update');
         $r->get('/r/:nonce/data')->to(controller => 'review', action => 'data');
-
-	$r->get('/review/:nonce' => sub {
-		my $c = shift;
-		my $stt = $c->dbh->prepare("SELECT state, name, id, extract(epoch from prelen) as prelen, extract(epoch from postlen) as postlen, extract(epoch from (endtime - starttime)) as length, speakers, starttime, endtime, slug, room, comments FROM talk_list WHERE nonce=?");
-		my $rv = $stt->execute($c->param("nonce"));
-		if($rv == 0) {
-			$c->res->code(404);
-			$c->render(text => "Invalid URL");
-			return undef;
-		}
-		my $row = $stt->fetchrow_hashref("NAME_lc");
-		if($row->{state} ne 'preview' && $row->{state} ne 'broken') {
-			$c->stash(message => "The talk <q>" . $row->{name} . "</q> is not currently available for review. It is in the state <tt>" . $row->{state} . "</tt>, whereas we need the <tt>preview</tt> state to do review. For more information, please see <a href='https://yoe.github.io/sreview/'>the documentation</a>");
-			$c->stash(title => 'Review finished or not yet available.');
-			$c->render('msg');
-			return undef;
-		}
-		my $stp = $c->dbh->prepare("SELECT properties.name, properties.description, properties.helptext, corrections.property_value FROM properties left join corrections on (properties.id = corrections.property AND talk = ?) ORDER BY properties.description");
-		$stp->execute($row->{id});
-		my $viddata = {};
-		$viddata->{corrvals} = {};
-		$viddata->{corrdescs} = {};
-		$viddata->{corrhelps} = {};
-		while(my $corrrow = $stp->fetchrow_hashref) {
-			$viddata->{corrdescs}{$corrrow->{name}} = $corrrow->{description};
-			$viddata->{corrhelps}{$corrrow->{name}} = $corrrow->{helptext};
-			$viddata->{corrvals}{$corrrow->{name}} = $corrrow->{property_value} + 0;
-		}
-
-		$viddata->{mainlen} = $row->{length} + 0;
-		$viddata->{prelen} = $row->{prelen} + 0;
-		$viddata->{postlen} = $row->{postlen} + 0;
-
-		$c->stash(title => 'Review for ' . $row->{name});
-		$c->stash(talk_title => $row->{name});
-		$c->stash(talk_speakers => $row->{speakers});
-		$c->stash(talk_start => $row->{starttime});
-		$c->stash(talk_end => $row->{endtime});
-		$c->stash(talk_nonce => $c->param("nonce"));
-		$c->stash(slug => $row->{slug});
-		$c->stash(event => $config->get("event"));
-		$c->stash(eventid => $c->eventid);
-		$c->stash(room => $row->{room});
-		$c->stash(state => $row->{state});
-		$c->stash(corrections => $viddata);
-		$c->stash(comments => $row->{comments});
-		$c->stash(target => "talk_update");
-		$c->stash(layout => 'default');
-		$c->stash(scripts_raw => ['sreview_viddata = ' . encode_json($viddata) . ';']);
-		$c->stash(scripts_extra => ['/mangler.js']);
-		$c->stash(exten => $config->get('preview_exten'));
-		$c->stash(vid_hostname => $config->get("vid_prefix"));
-	} => 'talk');
+	$r->get('/f/:nonce')->to(controller => 'finalreview', action => 'view', layout => 'default');
+	$r->post('/f/:nonce/update')->to(controller => 'finalreview', action => 'update', layout => 'default');
 
 	$r->get('/released' => sub {
 		my $c = shift;
@@ -225,7 +241,7 @@ sub startup {
 		$conference->{title} = $config->get("event");
 		my $row = $st->fetchrow_hashref();
 		$conference->{date} = [ $row->{min}, $row->{max} ];
-		$conference->{video_formats} = [];
+		$conference->{video_formats} = {};
 		$st = $c->dbh->prepare("SELECT filename FROM raw_files JOIN talks ON raw_files.room = talks.room WHERE talks.event = ? LIMIT 1");
 		$st->execute($c->eventid);
 		if($st->rows < 1) {
@@ -233,22 +249,30 @@ sub startup {
 			return;
 		}
 		$row = $st->fetchrow_hashref;
-		my $vid = SReview::Video->new(url => $row->{filename});
+		my $collection = SReview::Files::Factory->create(input => $config->get("inputglob"), $config);
+		my $vid = Media::Convert::Asset->new(url => $collection->get_file(relname => $row->{filename})->filename);
 		foreach my $format(@{$config->get("output_profiles")}) {
 			my $nf;
 			$self->log->debug("profile $format");
-			my $prof = SReview::Video::ProfileFactory->create($format, $vid);
+			my $prof = Media::Convert::Asset::ProfileFactory->create($format, $vid, $self->srconfig->get('extra_profiles'));
 			if(!$have_default) {
 				$nf = "default";
 				$have_default = 1;
 			} else {
 				$nf = $format;
 			}
-			push @{$conference->{video_formats}}, { $nf => { vcodec => $prof->video_codec, acodec => $prof->audio_codec, resolution => $prof->video_size, bitrate => $prof->video_bitrate } };
+			my $hash = { vcodec => $prof->video_codec, acodec => $prof->audio_codec, resolution => $prof->video_size, bitrate => $prof->video_bitrate };
+			if(!defined($hash->{bitrate})) {
+				$hash->{bitrate} = "";
+			}
+			if($hash->{bitrate} =~ /\d+/) {
+				$hash->{bitrate} = $hash->{bitrate} . "k";
+			}
+			$conference->{video_formats}{$nf} = $hash;
 			$formats{$nf} = $prof;
 		}
 		$json{conference} = $conference;
-		$st = $c->dbh->prepare("SELECT title, subtitle, speakerlist(talks.id), description, starttime, starttime::date AS date, to_char(starttime, 'yyyy') AS year, endtime, rooms.name AS room, rooms.outputname AS room_output, upstreamid, events.name AS event, slug FROM talks JOIN rooms ON talks.room = rooms.id JOIN events ON talks.event = events.id WHERE state='done' AND event = ?");
+		$st = $c->dbh->prepare("SELECT talks.id AS talkid, title, subtitle, description, starttime, starttime::date AS date, to_char(starttime, 'yyyy') AS year, endtime, rooms.name AS room, rooms.outputname AS room_output, upstreamid, events.name AS event, slug FROM talks JOIN rooms ON talks.room = rooms.id JOIN events ON talks.event = events.id WHERE state='done' AND event = ?");
 		$st->execute($c->eventid);
 		if($st->rows < 1) {
 			$c->render(json => {});
@@ -260,7 +284,7 @@ sub startup {
 			my $video = {};
 			my $subtitle = defined($row->{subtitle}) ? " " . $row->{subtitle} : "";
 			$video->{title} = $row->{title} . $subtitle;
-			$video->{speakers} = [ $row->{speakerlist} ];
+			$video->{speakers} = SReview::Talk->new(talkid => $row->{talkid})->speakerlist;
 			$video->{description} = $row->{description};
 			$video->{start} = $row->{starttime};
 			$video->{end} = $row->{endtime};
@@ -276,7 +300,10 @@ sub startup {
 					slug => $row->{slug},
 					room => $row->{room},
 					date => $row->{date},
+					event => $row->{event},
+					upstreamid => $row->{upstreamid},
 					year => $row->{year} });
+				chomp $video->{details_url};
 			}
 			$video->{video} = join('/',$outputdir, $row->{slug}) . "." . $formats{default}->exten;
 			push @$videos, $video;
@@ -286,56 +313,12 @@ sub startup {
 	});
 
 	$r->get('/overview' => sub {
-		my $c = shift;
-		my $st;
-		if($config->get("anonreviews")) {
-			$st = $c->dbh->prepare('SELECT nonce, name, speakers, room, starttime::timestamp, endtime::timestamp, state, progress FROM talk_list WHERE eventid = ? AND state IS NOT NULL ORDER BY state, progress, room, starttime');
-		} else {
-			$st = $c->dbh->prepare('SELECT name, speakers, room, starttime::timestamp, endtime::timestamp, state, progress FROM talk_list WHERE eventid = ? AND state IS NOT NULL ORDER BY state, progress, room, starttime');
-		}
-		my $tot = $c->dbh->prepare('SELECT state, count(*) FROM talks WHERE event = ? GROUP BY state ORDER BY state;');
-		my %expls;
-		my $tot_results;
-		my $totals = [];
-		$expls{'waiting_for_files'} = 'Still waiting for content files for these talks';
-		$expls{'cutting'} = 'Talk is being cut';
-		$expls{'generating_previews'} = 'Talk previews are being generated';
-		$expls{'notification'} = 'Sending out notifications';
-		$expls{'preview'} = 'Talk ready for review, waiting for reviewer';
-		$expls{'transcoding'} = 'High-quality transcodes running';
-		$expls{'uploading'} = 'Publishing results';
-		$expls{'announcing'} = 'Announcing completion of publication';
-		$expls{'done'} = 'Videos published, all done';
-		$expls{'broken'} = 'Review found problems, administrator required';
-		$expls{'ignored'} = 'Talk will not be/was not recorded, ignored for review';
-		$expls{'needs_work'} = 'Fixable problems exist, manual intervention required';
-		$expls{'lost'} = 'Nonfixable problems exist, talk lost';
-		$st->execute($c->eventid) or die;
-		$tot->execute($c->eventid) or die;
-		$c->stash(title => 'Video status overview');
-		$c->stash(titlerow => [ 'Talk', 'Speakers', 'Room', 'Start time', 'End time', 'State', 'Progress' ]);
-		$c->stash(tottitrow => [ 'State', 'Count', 'State meaning']);
-		my $rows = $st->fetchall_arrayref;
-		if($config->get("anonreviews")) {
-			my $newrows = [];
-			foreach my $row(@$rows) {
-				my $nonce = shift @$row;
-				my $name = shift @$row;
-				push @$newrows, [ "<a href='/r/$nonce'>$name</a>", @$row ];
-			}
-			$rows = $newrows;
-		}
-		$c->stash(rows => $rows);
-		$c->stash(header => 'Video status overview');
-		$tot_results = $tot->fetchall_arrayref();
-		foreach my $row(@{$tot_results}) {
-			push @$row, $expls{$row->[0]};
-			push @$totals, $row;
-		}
-		$c->stash(totals => $totals);
-		$c->stash(layout => 'default');
-		$c->render;
-	} => 'table');
+		shift->render;
+	});
+
+	$r->get("/credits" => sub {
+		shift->render;
+	});
 
 	$r->post('/talk_update' => sub {
 		my $c = shift;
@@ -394,84 +377,19 @@ sub startup {
 	$admin->any('/schedule/talk/')->to(controller => 'schedule', action => 'mod_talk');
 	$admin->any('/schedule/')->to(controller => 'schedule', action => 'index');
 
-	$admin->get('/')->to('admin#main');
+	$admin->get('/')->to('admin#main')->name("admin_talk");
 
 	$admin->get('/logout' => sub {
 		my $c = shift;
 		delete $c->session->{id};
 		delete $c->session->{room};
+		# Note, doesn't seem to work in chromium:
+		# https://bugs.chromium.org/p/chromium/issues/detail?id=696204
+		$c->cookie(sreview_api_key => '', {expires => 0});
 		$c->redirect_to('/');
 	});
 
-	$admin->get('/talk' => sub {
-		my $c = shift;
-		my $id = $c->param("talk");
-		my $st;
-
-		if(defined($c->session->{room})) {
-			$st = $c->dbh->prepare('SELECT state, name, id, extract(epoch from prelen) as prelen, extract(epoch from postlen) as postlen, extract(epoch from (endtime - starttime)) as length, speakers, starttime, endtime, slug, room, comments, apologynote, nonce FROM talk_list WHERE id = ? AND roomid = ?');
-			$st->execute($id, $c->session->{room});
-		} else {
-			$st = $c->dbh->prepare('SELECT state, name, id, extract(epoch from prelen) as prelen, extract(epoch from postlen) as postlen, extract(epoch from (endtime - starttime)) as length, speakers, starttime, endtime, slug, room, comments, apologynote, nonce FROM talk_list WHERE id = ?');
-			$st->execute($id);
-		}
-		my $row = $st->fetchrow_hashref("NAME_lc");
-		my $stp = $c->dbh->prepare("SELECT properties.name, properties.description, properties.helptext, corrections.property_value FROM properties left join corrections on (properties.id = corrections.property AND corrections.talk = ?) ORDER BY properties.description");
-		$stp->execute($id);
-
-		if(!defined($row)) {
-			$c->stash(message => "Unknown talk.");
-			$c->render('error');
-			return undef;
-		}
-		my $viddata = {};
-		$viddata->{corrvals} = {};
-		$viddata->{corrdescs} = {};
-		$viddata->{corrhelps} = {};
-		while(my $corrrow = $stp->fetchrow_hashref) {
-			$viddata->{corrdescs}{$corrrow->{name}} = $corrrow->{description};
-			$viddata->{corrvals}{$corrrow->{name}} = $corrrow->{property_value} + 0;
-			$viddata->{corrhelps}{$corrrow->{name}} = $corrrow->{helptext};
-		}
-		$viddata->{mainlen} = $row->{length} + 0;
-		$viddata->{prelen} = $row->{prelen} + 0;
-		$viddata->{postlen} = $row->{postlen} + 0;
-
-		$c->stash(talk_title => $row->{name});
-		$c->stash(talk_speakers => $row->{speakers});
-		$c->stash(talk_start => $row->{starttime});
-		$c->stash(talk_end => $row->{endtime});
-		$c->stash(talk_nonce => $row->{nonce});
-		$c->stash(slug => $row->{slug});
-		$c->stash(event => $config->get("event"));
-		$c->stash(eventid => $c->eventid);
-		$c->stash(room => $row->{room});
-		$c->stash(state => $row->{state});
-		$c->stash(comments => $row->{comments});
-		$c->stash(corrections => $viddata);
-		$c->stash(target => "talk_update_admin");
-		$c->stash(scripts_raw => ['sreview_viddata = ' . encode_json($viddata) . ';']);
-		$c->stash(scripts_extra => ['/mangler.js']);
-		$c->stash(type => "admin");
-		$c->stash(apology => $row->{apologynote});
-		$c->stash(vid_hostname => $config->get("vid_prefix"));
-		$c->stash(exten => $config->get('preview_exten'));
-		$c->render(template => 'talk');
-	} => 'admin_talk');
-
-	$admin->post('/talk_update' => sub {
-		my $c = shift;
-		my $talk = $c->param("talk");
-		if(!defined($talk)) {
-			$c->stash(message => "Required parameter talk missing.");
-			$c->render("error");
-			return undef;
-		}
-		$c->stash(template => 'talk');
-		$c->flash(completion_message => 'Your change has been accepted. Thanks for your help!');
-		$c->talk_update($talk);
-		$c->redirect_to("/admin/talk?talk=$talk");
-	} => 'talk_update_admin');
+	$admin->get('/talk')->to('review#view');
 
 	$admin->get('/brokens' => sub {
 		my $c = shift;
@@ -494,6 +412,7 @@ sub startup {
 		$c->stash(header => 'Broken talks');
 		$c->stash(layout => 'admin');
 		$c->stash(totals => undef);
+		$c->stash(autorefresh => 0);
 		$c->render(template => 'table');
 	} => 'broken_table');
 
