@@ -2,7 +2,7 @@ package SReview::Web::Controller::Talk;
 
 use Mojo::Base 'Mojolicious::Controller';
 use SReview::API::Helpers qw/db_query update_with_json add_with_json/;
-use Mojo::Util;
+use Mojo::Util 'slugify';
 use Mojo::JSON qw/encode_json decode_json/;
 use DateTime::Format::Pg;
 
@@ -20,6 +20,14 @@ sub format_talks {
 	return $talks;
 }
 
+sub fixup {
+	my $talk = shift;
+	if($talk->{flags}) {
+		$talk->{flags} = decode_json($talk->{flags});
+	}
+	return $talk;
+}
+
 sub listByEvent {
 	my $c = shift->openapi->valid_input or return;
 
@@ -54,8 +62,11 @@ sub add {
 
 	my $talk = $c->req->json;
 	$talk->{event} = $event->[0];
+	if(!exists($talk->{slug})) {
+		$talk->{slug} = slugify($talk->{title});
+	}
 
-	return add_with_json($c, $talk, "talks", $c->openapi->spec('/components/schemas/Talk/properties'));
+	return add_with_json($c, $talk, "talks", $c->openapi->spec('/components/schemas/Talk/properties'), \&fixup);
 }
 
 sub update {
@@ -80,7 +91,7 @@ sub update {
 		$talk->{flags} = encode_json($talk->{flags});
 	}
 
-	return update_with_json($c, $talk, "talks", $c->openapi->spec('/components/schemas/Talk/properties'));
+	return update_with_json($c, $talk, "talks", $c->openapi->spec('/components/schemas/Talk/properties'), \&fixup);
 }
 
 sub delete {
@@ -231,12 +242,7 @@ sub getByNonce {
 		$c->render(text => "not found");
 		return;
 	}
-	foreach my $r(@$talk) {
-		$r->{starttime} = DateTime::Format::Pg->parse_datetime($r->{starttime})->iso8601();
-		$r->{endtime} = DateTime::Format::Pg->parse_datetime($r->{endtime})->iso8601();
-	}
-
-	$c->render(openapi => $talk->[0]);
+	$c->render(openapi => format_talks($talk)->[0]);
 }
 
 sub getCorrections {
@@ -245,17 +251,36 @@ sub getCorrections {
 	my $eventId = $c->param("eventId");
 	my $talkId = $c->param("talkId");
 
-	$talkId = db_query("SELECT id FROM talks WHERE event = ? AND id = ?", $eventId, $talkId);
+	my $talk = db_query($c->dbh, "SELECT id FROM talks WHERE event = ? AND id = ?", $eventId, $talkId);
 
-	if(scalar(@$talkId) < 1) {
+	if(scalar(@$talk) < 1) {
 		$c->res->code(404);
 		$c->render(text => "event or talk not found");
 		return;
 	}
 
-	my $talk = SReview::Talk->new(talkid => $talkId);
+	$talk = SReview::Talk->new(talkid => $talkId);
 
 	$c->render(openapi => $talk->corrections);
 }
 
+sub getRelativeName {
+	my $c = shift->openapi->valid_input or return;
+
+	my $eventId = $c->param("eventId");
+	my $talkId = $c->param("talkId");
+
+	my $talk = db_query($c->dbh, "SELECT id FROM talks WHERE event = ? AND id = ?", $eventId, $talkId);
+
+	if(scalar(@$talk) < 1) {
+		$c->res->code(404);
+		$c->render(text => "event or talk not found");
+		return;
+	}
+
+	$talk = SReview::Talk->new(talkid => $talkId);
+
+	$c->render(openapi => $talk->relative_name);
+}
+
 1;
